@@ -28,17 +28,17 @@
   (This is the Modified BSD License)
 */
 
+// clang-format off
 
 /// \file
 /// An API for accessing images via a system that
 /// automatically manages a cache of resident image data.
 
 
-#ifndef OPENIMAGEIO_IMAGECACHE_H
-#define OPENIMAGEIO_IMAGECACHE_H
+#pragma once
 
-#include <OpenImageIO/ustring.h>
 #include <OpenImageIO/imageio.h>
+#include <OpenImageIO/ustring.h>
 
 
 // Define symbols that let client applications determine if newly added
@@ -56,8 +56,7 @@ namespace pvt {
 class ImageCacheImpl;
 class ImageCacheFile;
 class ImageCachePerThreadInfo;
-};
-
+};  // namespace pvt
 
 
 
@@ -73,19 +72,17 @@ public:
     /// If shared==true, it's intended to be shared with other like-minded
     /// owners in the same process who also ask for a shared cache.  If
     /// false, a private image cache will be created.
-    static ImageCache *create (bool shared=true);
+    static ImageCache* create(bool shared = true);
 
-    /// Destroy a ImageCache that was created using ImageCache::create().
-    /// The variety that takes a 'teardown' parameter, when set to true,
-    /// will fully destroy even a "shared" ImageCache.
-    static void destroy (ImageCache * x);
-    static void destroy (ImageCache * x, bool teardown);
+    /// Destroy an ImageCache that was created using ImageCache::create(),
+    /// unless it's recognized that this is the global "shared" ImageCache,
+    /// in which case it will stay active.
+    /// However, if 'teardown' is true, it will fully destroy the IC even
+    /// if it's the shared one (use with caution).
+    static void destroy(ImageCache* x, bool teardown = false);
 
-    ImageCache (void) { }
-    virtual ~ImageCache () { }
-
-    OIIO_DEPRECATED("clear() was never implemented. Don't bother calling it. [1.7]")
-    virtual void clear () { }
+    ImageCache(void) {}
+    virtual ~ImageCache() {}
 
     /// Set an attribute controlling the image cache.  Return true
     /// if the name and type were recognized and the attrib was set.
@@ -156,14 +153,14 @@ public:
     /// but passing in their managed pointer. If the passed-in thread_info
     /// is not NULL, it won't create a new one or retrieve a TSP, but it
     /// will do other necessary housekeeping on the Perthread information.
-    virtual Perthread * get_perthread_info (Perthread *thread_info = NULL) = 0;
+    virtual Perthread* get_perthread_info(Perthread* thread_info = NULL) = 0;
 
     /// Create a new Perthread. It is the caller's responsibility to
     /// eventually destroy it using destroy_thread_info().
-    virtual Perthread * create_thread_info () = 0;
+    virtual Perthread* create_thread_info() = 0;
 
     /// Destroy a Perthread that was allocated by create_thread_info().
-    virtual void destroy_thread_info (Perthread *thread_info) = 0;
+    virtual void destroy_thread_info(Perthread* thread_info) = 0;
 
     /// Define an opaque data type that allows us to have a handle to an
     /// image (already having its name resolved) but without exposing
@@ -179,11 +176,11 @@ public:
 
     /// Return true if the image handle (previously returned by
     /// get_image_handle()) is a valid image that can be subsequently read.
-    virtual bool good (ImageHandle *file) = 0;
+    virtual bool good(ImageHandle* file) = 0;
 
     /// Given possibly-relative 'filename', resolve it using the search
     /// path rules and return the full resolved filename.
-    virtual std::string resolve_filename (const std::string &filename) const=0;
+    virtual std::string resolve_filename(const std::string& filename) const = 0;
 
     /// Get information about the named image.  Return true if found
     /// and the data has been put in *data.  Return false if the image
@@ -307,50 +304,53 @@ public:
 
     /// After finishing with a tile, release_tile will allow it to
     /// once again be purged from the tile cache if required.
-    virtual void release_tile (Tile *tile) const = 0;
+    virtual void release_tile(Tile* tile) const = 0;
 
     /// Retrieve the data type of the pixels stored in the tile, which may
     /// be different than the type of the pixels in the disk file.
-    virtual TypeDesc tile_format (const Tile *tile) const = 0;
+    virtual TypeDesc tile_format(const Tile* tile) const = 0;
 
     /// Retrieve the ROI describing the pixels and channels stored in the
     /// tile.
-    virtual ROI tile_roi (const Tile *tile) const = 0;
+    virtual ROI tile_roi(const Tile* tile) const = 0;
 
     /// For a tile retrived by get_tile(), return a pointer to the
     /// pixel data itself, and also store in 'format' the data type that
     /// the pixels are internally stored in (which may be different than
     /// the data type of the pixels in the disk file).
-    virtual const void * tile_pixels (Tile *tile, TypeDesc &format) const = 0;
+    virtual const void* tile_pixels(Tile* tile, TypeDesc& format) const = 0;
 
     /// The add_file() call causes a file to be opened or added to the
     /// cache. There is no reason to use this method unless you are
     /// supplying a custom creator, or configuration, or both.
-    /// 
+    ///
     /// If creator is not NULL, it points to an ImageInput::Creator that
     /// will be used rather than the default ImageInput::create(), thus
     /// instead of reading from disk, creates and uses a custom ImageInput
     /// to generate the image. The 'creator' is a factory that creates the
     /// custom ImageInput and will be called like this:
-    ///      ImageInput *in = creator();
+    ///      std::unique_ptr<ImageInput> in (creator());
     /// Once created, the ImageCache owns the ImageInput and is responsible
     /// for destroying it when done. Custom ImageInputs allow "procedural"
     /// images, among other things.  Also, this is the method you use to set
     /// up a "writeable" ImageCache images (perhaps with a type of
     /// ImageInput that's just a stub that does as little as possible).
-    /// 
+    ///
     /// If config is not NULL, it points to an ImageSpec with configuration
     /// options/hints that will be passed to the underlying
     /// ImageInput::open() call. Thus, this can be used to ensure that the
     /// ImageCache opens a call with special configuration options.
-    /// 
+    ///
     /// This call (including any custom creator or configuration hints) will
     /// have no effect if there's already an image by the same name in the
     /// cache. Custom creators or configurations only "work" the FIRST time
     /// a particular filename is referenced in the lifetime of the
-    /// ImageCache.
-    virtual bool add_file (ustring filename, ImageInput::Creator creator=NULL,
-                           const ImageSpec *config=NULL) = 0;
+    /// ImageCache. But if replace is true, any existing entry will be
+    /// invalidated, closed and overwritten. So any subsequent access will see
+    /// the new file. Existing texture handles will still be valid.
+    virtual bool add_file (ustring filename, ImageInput::Creator creator=nullptr,
+                           const ImageSpec *config=nullptr,
+                           bool replace = false) = 0;
 
     /// Preemptively add a tile corresponding to the named image, at the
     /// given subimage, MIP level, and channel range.  The tile added is the
@@ -358,38 +358,32 @@ public:
     /// given format, with supplied strides) which will be copied and
     /// inserted into the cache and made available for future lookups.
     /// If chend < chbegin, it will add a tile containing the full set of
-    /// channels for the image.
+    /// channels for the image. Note that if the 'copy' flag is false, the
+    /// data is assumed to be in some kind of persistent storage and will
+    /// not be copied, nor will its pixels take up additional memory in the
+    /// cache.
     virtual bool add_tile (ustring filename, int subimage, int miplevel,
                      int x, int y, int z, int chbegin, int chend,
                      TypeDesc format, const void *buffer,
                      stride_t xstride=AutoStride, stride_t ystride=AutoStride,
-                     stride_t zstride=AutoStride) = 0;
-
-    OIIO_DEPRECATED("Use the version of add_tile with channel range. [1.6]")
-    virtual bool add_tile (ustring filename, int subimage, int miplevel,
-                     int x, int y, int z, TypeDesc format, const void *buffer,
-                     stride_t xstride=AutoStride, stride_t ystride=AutoStride,
-                     stride_t zstride=AutoStride) {
-        return add_tile (filename, subimage, miplevel, x, y, z, 0, -1,
-                         format, buffer, xstride, ystride, zstride);
-    }
+                     stride_t zstride=AutoStride, bool copy = true) = 0;
 
     /// If any of the API routines returned false indicating an error,
     /// this routine will return the error string (and clear any error
     /// flags).  If no error has occurred since the last time geterror()
     /// was called, it will return an empty string.
-    virtual std::string geterror () const = 0;
+    virtual std::string geterror() const = 0;
 
     /// Return the statistics output as a huge string.
     ///
-    virtual std::string getstats (int level=1) const = 0;
+    virtual std::string getstats(int level = 1) const = 0;
 
     /// Reset most statistics to be as they were with a fresh
     /// ImageCache.  Caveat emptor: this does not flush the cache itelf,
     /// so the resulting statistics from the next set of texture
     /// requests will not match the number of tile reads, etc., that
     /// would have resulted from a new ImageCache.
-    virtual void reset_stats () = 0;
+    virtual void reset_stats() = 0;
 
     /// Invalidate any loaded tiles or open file handles associated with
     /// the filename, so that any subsequent queries will be forced to
@@ -401,7 +395,7 @@ public:
     /// reference-counted tile pointers from the named image, but those
     /// procedures will not get updated pixels until they release the
     /// tiles they are holding.
-    virtual void invalidate (ustring filename) = 0;
+    virtual void invalidate(ustring filename) = 0;
 
     /// Invalidate all loaded tiles and open file handles.  This is safe
     /// to do even if other procedures are currently holding
@@ -412,7 +406,7 @@ public:
     /// false, in actuality files will only be invalidated if their
     /// modification times have been changed since they were first
     /// opened.
-    virtual void invalidate_all (bool force=false) = 0;
+    virtual void invalidate_all(bool force = false) = 0;
 
     /// Close any open file handles associated with a named file, or for all
     /// files, but do not invalidate any image spec information or pixels
@@ -425,10 +419,8 @@ public:
 private:
     // Make delete private and unimplemented in order to prevent apps
     // from calling it.  Instead, they should call ImageCache::destroy().
-    void operator delete (void * /*todel*/) { }
+    void operator delete(void* /*todel*/) {}
 };
 
 
 OIIO_NAMESPACE_END
-
-#endif // OPENIMAGEIO_IMAGECACHE_H
